@@ -2,6 +2,7 @@
 #
 # This file is part of gaffer. See the NOTICE for more information.
 
+import copy
 import socket
 
 # patch tornado IOLoop
@@ -16,6 +17,31 @@ from tornado.httpserver import HTTPServer
 from .util import parse_address, is_ipv6
 from . import http
 
+
+DEFAULT_HANDLERS = [
+        (r'/', http.WelcomeHandler),
+        (r'/processes', http.ProcessesHandler),
+        (r'/processes/([0-9^/]+)', http.ProcessIdHandler),
+        (r'/processes/([^/]+)', http.ProcessHandler),
+        (r'/processes/([0-9^/]+)/(_[^/]+)$', http.ProcessIdManageHandler),
+        (r'/processes/([^/]+)/(_[^/]+)$', http.ProcessManagerHandler),
+        (r'/processes/([0-9^/]+)/(_[^/]+)/(.*)$', http.ProcessIdManageHandler),
+        (r'/processes/([^/]+)/(_[^/]+)/(.*)$', http.ProcessManagerHandler),
+        (r'/status/([^/]+)', http.StatusHandler),
+        (r'/watch', http.WatcherHandler),
+        (r'/watch/([^/]+)$', http.WatcherHandler),
+        (r'/watch/([^/]+)/([^/]+)$', http.WatcherHandler),
+        (r'/watch/([^/]+)/([^/]+)/([^/]+)$', http.WatcherHandler),
+        (r'/stats', http.StatsHandler),
+        (r'/stats/([^/]+)', http.StatsHandler),
+        (r'/stats/([^/]+)/([0-9^/]+)$', http.StatsHandler),
+        (r'/streams/([0-9^/]+)/([^/]+)$', http.StreamHandler),
+        (r'/wstreams/([0-9^/]+)$', http.WStreamHandler),
+        (r'/groups', http.GroupsHandler),
+        (r'/groups/([^/]+)$', http.GroupHandler),
+        (r'/groups/([^/]+)/(_[^/]+)$', http.GroupHandler),
+]
+
 class HttpEndpoint(object):
 
     def __init__(self, uri='127.0.0.1:5000', backlog=128,
@@ -66,6 +92,7 @@ class HttpEndpoint(object):
 
     def stop(self):
         self.server.stop()
+        self.io_loop.close(True)
 
     def restart(self):
         self.server.stop()
@@ -78,34 +105,13 @@ class HttpHandler(object):
     sockets) with different options. Each endpoint can also listen on
     different interfaces """
 
-    DEFAULT_HANDLERS = [
-            (r'/', http.WelcomeHandler),
-            (r'/processes', http.ProcessesHandler),
-            (r'/processes/([0-9^/]+)', http.ProcessIdHandler),
-            (r'/processes/([^/]+)', http.ProcessHandler),
-            (r'/processes/([0-9^/]+)/(_[^/]+)$', http.ProcessIdManageHandler),
-            (r'/processes/([^/]+)/(_[^/]+)$', http.ProcessManagerHandler),
-            (r'/processes/([0-9^/]+)/(_[^/]+)/(.*)$', http.ProcessIdManageHandler),
-            (r'/processes/([^/]+)/(_[^/]+)/(.*)$', http.ProcessManagerHandler),
-            (r'/status/([^/]+)', http.StatusHandler),
-            (r'/watch', http.WatcherHandler),
-            (r'/watch/([^/]+)$', http.WatcherHandler),
-            (r'/watch/([^/]+)/([^/]+)$', http.WatcherHandler),
-            (r'/watch/([^/]+)/([^/]+)/([^/]+)$', http.WatcherHandler),
-            (r'/stats', http.StatsHandler),
-            (r'/stats/([^/]+)', http.StatsHandler),
-            (r'/stats/([^/]+)/([0-9^/]+)$', http.StatsHandler),
-            (r'/streams/([0-9^/]+)/([^/]+)$', http.StreamHandler),
-            (r'/wstreams/([0-9^/]+)$', http.WStreamHandler)
-    ]
-
     def __init__(self, endpoints=[], handlers=None):
         self.endpoints = endpoints or []
         if not endpoints: # if no endpoints passed add a default
             self.endpoints.append(HttpEndpoint())
 
         # set http handlers
-        self.handlers = self.DEFAULT_HANDLERS
+        self.handlers = copy.copy(DEFAULT_HANDLERS)
         self.handlers.extend(handlers or [])
 
     def start(self, loop, manager):
