@@ -39,6 +39,7 @@ import pyuv
 import six
 from tornado import httpclient
 
+from .events import EventEmitter
 from .tornado_pyuv import IOLoop
 
 if six.PY3:
@@ -91,6 +92,7 @@ class HTTPClient(object):
         """Closes the HTTPClient, freeing any resources used."""
         if not self._closed:
             self._async_client.close()
+            self._io_loop.close(True)
             self._closed = True
 
     def fetch(self, request, **kwargs):
@@ -112,11 +114,107 @@ class HTTPClient(object):
         response.rethrow()
         return response
 
+class EventsourceClient(object):
+    """ simple client to fetch Gaffer streams using the eventsource
+    stream.
+
+    Example of usage::
+
+        loop = pyuv.Loop.default_loop()
+
+        def cb(event, data):
+            print(data)
+
+        # create a client
+        url = http://localhost:5000/streams/1/stderr?feed=continuous'
+        client = EventSourceClient(loop, url)
+
+        # subscribe to the stderr event
+        client.subscribe("stderr", cb)
+
+        # start the client
+        client.start()
+
+    """
+
+    def __init__(self, loop, url, **kwargs):
+        self.loop = loop
+        self._io_loop = IOLoop(_loop=loop)
+        self.url = url
+        self._emitter = EventEmitter(self.loop)
+        self.client = httpclient.AsyncHTTPClient(self._io_loop, **kwargs)
+        self.active = False
+        self.stopped = False
+
+    def start(self):
+        self.active = True
+        headers = {"Content-Type": "text/event-stream"}
+        req = httpclient.HTTPRequest(url=self.url,
+                                        method='GET',
+                                        headers=headers,
+                                        request_timeout=0,
+                                        streaming_callback=self._on_stream)
+
+        self.client.fetch(req, self._on_request)
+        self._io_loop.start(False)
+
+    def subscribe(self, event, listener):
+        self._emitter.subscribe(event, listener)
+
+    def unsubscribe(self, event, listener):
+        self._emitter.unsubscribe(event, listener)
+
+    def subscribe_once(self, event, listener):
+        self._emitter.subscribe_once(event, listener)
+
+    def render(self, event, data):
+        return data
+
+    def stop(self):
+        self.active = False
+        #self._emitter.close()
+        self.client.close()
+        self._io_loop.stop()
+        self._io_loop.close(True)
+
+    def run(self):
+        self.loop.run()
+
+    def _on_request(self, response):
+        self.stop()
+
+    def _on_stream(self, message):
+        if not message:
+            return
+        lines = [line for line in message.strip(b'\r\n').split(b"\r\n")]
+
+        event = None
+        data = []
+        for line in lines:
+            f, val = line.split(b":", 1)
+            if f == b"event":
+                event = val.strip()
+            elif f == b"data":
+                data.append(val.strip())
+        if event is None:
+            return
+
+        event = event.decode('utf-8')
+        data = self.render(event, b"\n".join(data).strip())
+        self._emitter.publish(event, data)
+
+class Watcher(EventsourceClient):
+    """ simple EventsourceClient wrapper that decode the JSON to a
+    python object """
+
+    def render(self, event, data):
+        return json.loads(data.decode('utf-8'))
+
 class Server(object):
     """ Server, main object to connect to a gaffer node. Most of the
     calls are blocking. (but running in the loop) """
 
-    def __init__(self, uri=None, loop=None, **options):
+    def __init__(self, uri, loop=None, **options):
         self.loop = loop or pyuv.Loop()
         self.uri = uri
         self.options = options
@@ -223,6 +321,7 @@ class Server(object):
         process.update(kwargs)
 
         body = json.dumps(process)
+
         headers = {"Content-Type": "application/json" }
 
         if force:
@@ -275,6 +374,42 @@ class Server(object):
             signum))
         return True
 
+    def groups(self):
+        """ return the list of all groups """
+        resp = self.request("get", "/groups")
+        return self.json_body(resp)
+
+    def get_group(self, name):
+        """ return the list of all process templates of this group """
+        resp = self.request("get", "/groups/%s" % name)
+        return self.json_body(resp)
+
+    def start_group(self, name):
+        """ start all process templates of the group """
+        self.request("post", "/groups/%s/_start" % name)
+        return True
+
+    def stop_group(self, name):
+        """ stop all process templates of the group """
+        self.request("post", "/groups/%s/_stop" % name)
+        return True
+
+
+    def restart_group(self, name):
+        """ restart all process templates of the group """
+        self.request("post", "/groups/%s/_restart" % name)
+        return True
+
+    def remove_group(self, name):
+        """ remove the group and all process templates of the group """
+        self.request("delete", "/groups/%s" % name)
+        return True
+
+    def get_watcher(self, heartbeat="true"):
+        """ return a watcher to listen on /watch """
+        url =  make_uri(self.uri, '/watch', feed='eventsource',
+                heartbeat=str(heartbeat))
+        return Watcher(self.loop, url, **self.options)
 
 class ProcessId(object):
     """ Process Id object. It represent a pid """
@@ -426,6 +561,12 @@ class Process(object):
         obj = self.server.json_body(resp)
         return obj['numprocesses']
 
+    def stats(self):
+        resp = self.server.request("get", "/stats/%s" % self.process['name'])
+        return self.server.json_body(resp)
+
+
+
     def signal(self, num_or_str):
         """ send a signal to all processes of this template """
         if isinstance(num_or_str, six.string_types):
