@@ -60,6 +60,7 @@ class ProcessTracker(object):
                     # done callback has been set, run it
                     if self._done_cb is not None:
                         self._done_cb()
+                        self._donc_cb = None
 
                     # nothing in the queue, quit
                     break
@@ -147,6 +148,10 @@ class ProcessState(object):
     def graceful_timeout(self):
         return nanotime(self.settings.get('graceful_timeout', 10.0))
 
+    @property
+    def group(self):
+        return self.settings.get('group')
+
     def __str__(self):
         return "state: %s" % self.name
 
@@ -226,6 +231,7 @@ class ProcessState(object):
         lcpu = []
         for p in self.running:
             info = p.info
+            info['id'] = p.id
             infos.append(info)
             lmem.append(info['mem'])
             lcpu.append(info['cpu'])
@@ -263,17 +269,20 @@ class ProcessState(object):
     def check_flapping(self):
         """ main function used to check the flapping """
         f = self.flapping
-        if len(f.history) < f.attempts:
-            f.history.append(time.time())
-        else:
+
+        f.history.append(time.time())
+        if len(f.history) >= f.attempts:
             diff = f.history[-1] - f.history[0]
             if diff > f.window:
                 f.reset()
-                f.history.append(time.time())
+                self.flapping = f
             elif f.retries < f.max_retry:
-                increment(f.retries)
+                f.retries = increment(f.retries)
+                self.flapping = f
                 return False, True
             else:
                 f.reset()
+                self.flapping = f
                 return False, False
+        self.flapping = f
         return True, None
