@@ -15,73 +15,6 @@ from gaffer.manager import Manager
 from gaffer.process import Process
 from gaffer.state import FlappingInfo
 
-class DummyProcess(object):
-
-    QUIT_SIGNALS = (signal.SIGQUIT, signal.SIGTERM,)
-
-    def __init__(self, testfile):
-        self.alive = True
-        self.testfile = testfile
-        self.queue = deque()
-        signal.signal(signal.SIGHUP, self.handle)
-        signal.signal(signal.SIGQUIT, self.handle)
-        signal.signal(signal.SIGTERM, self.handle)
-        signal.signal(signal.SIGCHLD, self.handle_chld)
-        signal.signal(signal.SIGWINCH, self.handle_winch)
-
-    def _write(self, msg):
-        with open(self.testfile, 'a+') as f:
-            f.write(msg)
-            f.flush()
-
-    def handle(self, signum, frame):
-        self.queue.append(signum)
-
-    def handle_quit(self):
-        self._write('QUIT')
-        self.alive = False
-
-    def handle_chld(self, *args):
-        self._write('CHLD')
-
-    def handle_winch(self, *args):
-        return
-
-    def handle_hup(self):
-        self._write('HUP')
-
-    def run(self):
-        self._write('START')
-
-        # write to std
-        sys.stdout.write("hello out")
-        sys.stdout.flush()
-        sys.stderr.write("hello err")
-        sys.stderr.flush()
-
-        while self.alive:
-            sig = None
-            try:
-                sig = self.queue.popleft()
-            except IndexError:
-                pass
-
-            if sig is not None:
-                if sig in self.QUIT_SIGNALS:
-                    self.handle_quit()
-                elif sig == signal.SIGHUP:
-                    self.handle_hup()
-
-            time.sleep(0.001)
-
-        self._write('STOP')
-
-
-def run_dummy(test_file):
-    dummy = DummyProcess(test_file)
-    dummy.run()
-    return 1
-
 def tmpfile():
      fd, testfile = mkstemp()
      os.close(fd)
@@ -90,73 +23,28 @@ def tmpfile():
 def dummy_cmd():
     fd, testfile = mkstemp()
     os.close(fd)
-    cmd = sys.executable
-    args = ['generic.py', "test_manager.run_dummy", testfile]
+    if sys.platform == 'win32':
+        cmd = "cmd.exe"
+        args = args=["/c", "proc_dummy.py", testfile],
+    else:
+        cmd = sys.executable
+        args = ["-u", "./proc_dummy.py", testfile]
     wdir = os.path.dirname(__file__)
     return (testfile, cmd, args, wdir)
 
 
-class CrashProcess(object):
-
-    def __init__(self):
-        self.alive = True
-
-    def run(self):
-        while self.alive:
-            time.sleep(0.1)
-            break
-
-def run_crashprocess():
-    c = CrashProcess()
-    c.run()
-    return 1
-
-
 def crash_cmd():
-    cmd = sys.executable
-    args = ['generic.py', "test_manager.run_crashprocess"]
-    wdir = os.path.dirname(__file__)
-    return (cmd, args, wdir)
-
-class EchoProcess(object):
-
-    def __init__(self, *args):
-        self.alive = True
-        signal.signal(signal.SIGQUIT, self.handle_quit)
-        signal.signal(signal.SIGTERM, self.handle_quit)
-        signal.signal(signal.SIGINT, self.handle_quit)
-        signal.signal(signal.SIGCHLD, self.handle_chld)
-
-    def handle_quit(self, *args):
-        self.alive = False
-
-    def handle_chld(self, *args):
-        pass
-
-
-
-    def run(self):
-        while self.alive:
-            c = sys.stdin.readline()
-            print(c)
-            sys.stdout.flush()
-            if c == "exit\n":
-                break
-
-def run_echocmd():
-    c = EchoProcess()
-    c.run()
-    return 1
-
-def echo_cmd():
     fd, testfile = mkstemp()
     os.close(fd)
-    cmd = sys.executable
-    args = ['generic.py', "test_manager.run_echocmd", testfile]
+    if sys.platform == 'win32':
+        cmd = "cmd.exe"
+        args = args=["/c", "proc_crash.py"],
+    else:
+        cmd = sys.executable
+        args = ["-u", "./proc_crash.py"]
     wdir = os.path.dirname(__file__)
     return (cmd, args, wdir)
 
-
 def test_simple():
     m = Manager()
     m.start()
@@ -378,30 +266,27 @@ def test_send_signal():
 def test_flapping():
     m = Manager()
     m.start()
+    states = []
     cmd, args, wdir = crash_cmd()
-    flapping = FlappingInfo(attempts=1, window=1, retry_in=0.1, max_retry=2)
+    flapping = FlappingInfo(attempts=1, window=1., retry_in=0.1,
+            max_retry=1)
     m.add_process("crashing", cmd, args=args, cwd=wdir, flapping=flapping)
-    state = m.get_process_state("crashing")
+    m.add_process("crashing2", cmd, args=args, cwd=wdir)
+    time.sleep(0.2)
 
     def cb(handle):
-        handle.stop()
-        assert state.stopped == True
+        state = m.get_process("crashing")
+        states.append(state.stopped)
+        state2 = m.get_process("crashing2")
+        states.append(state2.stopped)
+        m.stop()
 
     t = pyuv.Timer(m.loop)
-    t.start(cb, 0.8, 0.8)
-    m.add_process("crashing2", cmd, args=args, cwd=wdir)
-    state = m.get_process_state("crashing2")
-
-    def cb1(handle):
-        handle.stop()
-        assert state.stopped == False
-
-    t = pyuv.Timer(m.loop)
-    t.start(cb1, 0.8, 0.8)
-
-    m.stop()
+    t.start(cb, 0.8, 0.0)
     m.run()
 
+    assert states == [True, False]
+
 def test_events():
     emitted = []
     m = Manager()
@@ -473,7 +358,7 @@ def test_process_exit_event():
     m.run()
 
     assert len(emitted) == 1
-    assert len(emitted[0]) == 5
+    assert len(emitted[0]) == 6
 
     msg = emitted[0]
     assert "exit_status" in msg
@@ -506,7 +391,6 @@ def test_processes_stats():
     m.add_process("a", cmd, args=args, cwd=wdir)
     time.sleep(0.2)
     infos = list(m.processes_stats())
-    pid = m.running[1].pid
     m.add_process("b", cmd, args=args, cwd=wdir)
     infos2 = list(m.processes_stats())
     m.stop()
@@ -545,3 +429,70 @@ def test_monitor():
     assert res[0] == "stat"
     assert "cpu" in res[1]
     assert res[1]["pid"] == pid
+
+def test_priority():
+    m = Manager()
+    started = []
+    def cb(evtype, info):
+        started.append(info['name'])
+
+    m.start()
+    m.subscribe('start', cb)
+
+    testfile, cmd, args, wdir = dummy_cmd()
+    m.add_process("a", cmd, args=args, cwd=wdir, start=False)
+    m.add_process("d", cmd, args=args, cwd=wdir, start=False)
+    m.add_process("b", cmd, args=args, cwd=wdir, start=False)
+    m.start_processes()
+    def stop(handle):
+        m.unsubscribe("start", cb)
+        m.stop()
+
+    t = pyuv.Timer(m.loop)
+    t.start(stop, 0.4, 0.0)
+    m.run()
+
+    assert started == ["a", "d", "b"]
+
+def test_group():
+    m = Manager()
+    started = []
+    stopped = []
+    def cb(evtype, info):
+        if evtype == "start":
+            started.append(info['name'])
+        elif evtype == "stop":
+            stopped.append(info['name'])
+
+    m.start()
+    m.subscribe('start', cb)
+    m.subscribe('stop', cb)
+    testfile, cmd, args, wdir = dummy_cmd()
+    m.add_process("ga:a", cmd, args=args, cwd=wdir, start=False)
+    m.add_process("ga:b", cmd, args=args, cwd=wdir, start=False)
+    m.add_process("gb:a", cmd, args=args, cwd=wdir, start=False)
+    groups = sorted(m.get_groups())
+    ga1 = m.get_group('ga')
+    gb1 = m.get_group('gb')
+    m.start_group("ga")
+    m.stop_group("ga")
+    time.sleep(0.2)
+    m.remove_process("ga:a")
+    ga2 = m.get_group('ga')
+    m.stop_group("gb")
+
+    def stop(handle):
+        m.unsubscribe("start", cb)
+        m.unsubscribe("stop", cb)
+        m.stop()
+
+    t = pyuv.Timer(m.loop)
+    t.start(stop, 0.6, 0.0)
+    m.run()
+
+    assert groups == ['ga', 'gb']
+    assert ga1 == ['ga:a', 'ga:b']
+    assert gb1 == ['gb:a']
+    assert started == ['ga:a', 'ga:b']
+    assert stopped == ['ga:a', 'ga:b', 'gb:a']
+    assert ga2 == ['ga:b']
